import os
import shutil


from distutils.command.build_ext import build_ext
from distutils.core import Distribution
from distutils.core import Extension
from distutils.errors import CCompilerError
from distutils.errors import DistutilsExecError
from distutils.errors import DistutilsPlatformError


class get_pybind_include(object):
    def __str__(self):
        import pybind11
        return pybind11.get_include()


extensions = [
    Extension('relaxation', sources=['src/fretraj/relaxation.cpp'], include_dirs=[get_pybind_include()], language='c++')
    ]


class ExtBuilder(build_ext):

    built_extensions = []

    def run(self):
        try:
            build_ext.run(self)
        except (DistutilsPlatformError, FileNotFoundError):
            print('Unable to build C++ extensions of FRETraj')

    def build_extension(self, ext):
        try:
            build_ext.build_extension(self, ext)
        except (CCompilerError, DistutilsExecError, DistutilsPlatformError, ValueError):
            print(f'Unable to build the {ext.name} C++ extension, the burst module will not be available')


def build(setup_kwargs):

    distribution = Distribution({"name": "FRETraj", "ext_modules": extensions, "package_dir": "fretraj"})

    cmd = ExtBuilder(distribution)
    cmd.ensure_finalized()
    cmd.run()

    for output in cmd.get_outputs():
        if not os.path.exists(output):
            continue
        relative_extension = os.path.relpath(output, cmd.build_lib)
        destination = os.path.join('src', distribution.package_dir, relative_extension)
        shutil.copyfile(output, destination)
        mode = os.stat(destination).st_mode
        mode |= (mode & 0o444) >> 2
        os.chmod(destination, mode)

    return setup_kwargs


if __name__ == "__main__":
    build({})



#!/usr/bin/env python3
import pytest
from fretraj import _burst_module_found
import os
import jsonschema

if _burst_module_found:
    from fretraj import burst
else:
    pytestmark = pytest.mark.skip


_TEST_DIR = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))

BURST_PARAMETERS = {
    "dyes": {"tauD": 0.75, "tauA": 1.5, "QD": 0.2, "QA": 0.3},
    "sampling": {"nbursts": 2000, "skipframesatstart": 0, "skipframesatend": 1000, "multiprocessing": True},
    "fret": {"R0": 54, "kappasquare": 0.666666, "no_gamma": True, "quenching_radius": 1.0},
    "species": {
        "name": ["all"],
        "unix_pattern_rkappa": ["*.dat"],
        "unix_pattern_don_coords": ["*donor*.xvg"],
        "unix_pattern_acc_coords": ["*acceptor*.xvg"],
        "probability": [1],
    },
    "bursts": {"lower_limit": 20, "upper_limit": 150, "lambda": -2.3, "QY_correction": False, "averaging": "all"},
}


@pytest.fixture
def patch_open(mocker):
    mocker.patch("fretraj.burst.open")
    mocker.patch("json.load", return_value=BURST_PARAMETERS)


@pytest.fixture
def validate_parameters(patch_open):
    return burst.readParameters("anyname.json")


def test_donor_lifetime(validate_parameters):
    assert validate_parameters["dyes"]["tauD"] == 0.75


def test_missing_key(validate_parameters, monkeypatch):
    monkeypatch.delitem(BURST_PARAMETERS["dyes"], "tauD")
    with pytest.raises(jsonschema.exceptions.ValidationError):
        burst.readParameters("anyname.json")


def test_wrong_type(validate_parameters, monkeypatch):
    monkeypatch.setitem(BURST_PARAMETERS["dyes"], "tauD", "0.75")
    with pytest.raises(jsonschema.exceptions.ValidationError):
        burst.readParameters("anyname.json")


def testTrajectory_rkappa():
    traj = burst.Trajectory.from_file(rkappa_filename=os.path.join(_TEST_DIR, "data", "test_rkappa1.dat"))
    assert traj.time[0] == 0


def testTrajectory_rkappa_dyecoords():
    traj = burst.Trajectory.from_file(
        rkappa_filename=os.path.join(_TEST_DIR, "data", "test_rkappa1.dat"),
        don_coords_filename=os.path.join(_TEST_DIR, "data", "test_donorcoords1.xvg"),
        acc_coords_filename=os.path.join(_TEST_DIR, "data", "test_acceptorcoords1.xvg"),
    )
    assert traj.time[0] == 0


def testSpecies():
    species = burst.Species(
        name="all",
        probability=1,
        filelist_rkappa=[
            os.path.join(_TEST_DIR, "data", "test_rkappa1.dat"),
            os.path.join(_TEST_DIR, "data", "test_rkappa2.dat"),
        ],
    )
    assert species.name == "all"
    assert species.trajectories[0].time[0] == 0
    assert pytest.approx(species.trajectories[0].weight, abs=0.01) == 0.4


def testEnsemble():
    ensemble = burst.Ensemble(
        os.path.join(_TEST_DIR, "data"),
        {
            "species": {
                "name": ["all"],
                "unix_pattern_rkappa": ["*.dat"],
                "probability": [1],
                "n_trajectory_splits": None,
            }
        },
    )
    assert len(ensemble.species[0].trajectories) == 2


if __name__ == "__main__":
    pytest.main()



#!/usr/bin/env python3

import pytest
from fretraj import _burst_module_found
import os
import numpy as np

if _burst_module_found:
    from fretraj import burst
else:
    pytestmark = pytest.mark.skip

_TEST_DIR = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))


@pytest.fixture
def setup_parameters(mocker):
    BURST_PARAMETERS = {
        "dyes": {"tauD": 0.75, "tauA": 1.5, "QD": 0.2, "QA": 0.3},
        "sampling": {"nbursts": 2000, "skipframesatstart": 0, "skipframesatend": 1000, "multiprocessing": True},
        "fret": {"R0": 54, "kappasquare": 0.666666, "no_gamma": True, "quenching_radius": 1.0},
        "species": {
            "name": ["all"],
            "unix_pattern_rkappa": ["*.dat"],
            "unix_pattern_don_coords": ["*donor*.xvg"],
            "unix_pattern_acc_coords": ["*acceptor*.xvg"],
            "probability": [1],
        },
        "bursts": {"lower_limit": 20, "upper_limit": 150, "lambda": -2.3, "QY_correction": False, "averaging": "all"},
    }
    mocker.patch("fretraj.burst.open")
    mocker.patch("json.load", return_value=BURST_PARAMETERS)
    return burst.readParameters("anyname.json")


def test_singleCore_withoutAnisotropy(setup_parameters):
    setup_parameters["sampling"]["multiprocessing"] = False
    exp = burst.Experiment(
        os.path.join(_TEST_DIR, "data"),
        setup_parameters,
        binwidth=0.025,
        verbose=False,
        compute_anisotropy=False,
        units="nm",
    )
    assert pytest.approx(np.mean(exp.FRETefficiencies), abs=0.03) == 0.3


def test_multiCore_withoutAnisotropy(setup_parameters):
    exp = burst.Experiment(
        os.path.join(_TEST_DIR, "data"),
        setup_parameters,
        binwidth=0.025,
        verbose=False,
        compute_anisotropy=False,
        units="nm",
    )
    assert pytest.approx(np.mean(exp.FRETefficiencies), abs=0.03) == 0.3


def test_multiCore_withAnisotropy(setup_parameters):
    exp = burst.Experiment(
        os.path.join(_TEST_DIR, "data"),
        setup_parameters,
        binwidth=0.025,
        verbose=False,
        compute_anisotropy=True,
        units="nm",
    )
    assert pytest.approx(np.mean(exp.FRETefficiencies), abs=0.03) == 0.3


if __name__ == "__main__":
    pytest.main()
